try:
    import subprocess32 as sp
except ModuleNotFoundError:
    import subprocess as sp
import shlex

def run_command(cmd):
    p = sp.run(shlex.split(cmd), stdout=sp.PIPE, stderr=sp.PIPE)
    return p.stdout, p.stderr



#!/usr/bin/env python
import lz4.frame

try:
    import cPickle as pickle
except ImportError:
    import pickle

def main():
    with lz4.frame.open("task.p.lz4", 'rb') as f:
        task = pickle.load(f)

    result = task["task"](*task["args"], **task["kwargs"])

    with lz4.frame.open("result.p.lz4", 'wb') as f:
        pickle.dump(result, f)

if __name__ == "__main__":
    main()



import os
import logging
import lz4.frame
import time
import copy
from tqdm.auto import tqdm
from .utils import run_command
logger = logging.getLogger(__name__)

try:
    import cPickle as pickle
except ImportError:
    import pickle

SGE_JOBSTATUS = {
    1: "Running",
    2: "Pending",
    3: "Suspended",
    4: "Error",
    5: "Deleted",
    6: "Finished",
}

# https://gist.github.com/cmaureir/4fa2d34bc9a1bd194af1
SGE_JOBSTATE_CODES = {
    # Running
    "r": 1,
    "t": 1,
    "Rr": 1,
    "Rt": 1,

    # Pending
    "qw": 2,
    "hqw": 2,
    "hRwq": 2,

    # Suspended
    "s": 3, "ts": 3,
    "S": 3, "tS": 3,
    "T": 3, "tT": 3,
    "Rs": 3, "Rts":3, "RS":3, "RtS":3, "RT":3, "RtT": 3,

    # Error
    "Eqw": 4, "Ehqw": 4, "EhRqw": 4,

    # Deleted
    "dr": 5, "dt": 5, "dRr": 5, "ds": 5, "dS": 5, "dT": 5, "dRs": 5, "dRS": 5, "dRT": 5,
}

class JobMonitor(object):
    def __init__(self, submitter):
        self.submitter = submitter

    def monitor_jobs(self, sleep=5, request_user_input=True):
        jobid_tasks = self.submitter.jobid_tasks
        ntotal = len(jobid_tasks)

        pbar_run = tqdm(total=ntotal, desc="Running ")
        pbar_fin = tqdm(total=ntotal, desc="Finished")

        for running, results in self.return_finished_jobs(request_user_input=request_user_input):
            pbar_run.n = len(running)
            pbar_fin.n = len([r for r  in results if r is not None])
            pbar_run.refresh()
            pbar_fin.refresh()
            time.sleep(sleep)

        pbar_run.close()
        pbar_fin.close()
        print("")
        return results

    def request_jobs(self, sleep=5, request_user_input=True):
        jobid_tasks = self.submitter.jobid_tasks
        ntotal = len(jobid_tasks)

        pbar_run = tqdm(total=ntotal, desc="Running ")
        pbar_fin = tqdm(total=ntotal, desc="Finished")
        try:
            for running, results in self.return_finished_jobs(request_user_input=request_user_input):
                pbar_run.n = len(running)
                pbar_fin.n = len([r for r  in results if r is not None])
                pbar_run.refresh()
                pbar_fin.refresh()
                time.sleep(sleep)
                yield results
        except KeyboardInterrupt as e:
            self.submitter.killall()

        pbar_run.close()
        pbar_fin.close()
        print("")
        yield results

    def return_finished_jobs(self, request_user_input=True):
        jobid_tasks = self.submitter.jobid_tasks
        ntotal = len(jobid_tasks)
        nremaining = ntotal

        finished, results = [], [None]*ntotal

        while nremaining>0:
            job_statuses = self.query_jobs()
            all_queried_jobs = []
            for state, queried_jobs in job_statuses.items():
                all_queried_jobs.extend(queried_jobs)

            jobs_not_queried = {
                jobid: task
                for jobid, task in self.submitter.jobid_tasks.items()
                if jobid not in all_queried_jobs and jobid not in finished
            }
            finished.extend(self.check_jobs(jobs_not_queried, results, request_user_input=request_user_input))

            nremaining = ntotal - len(finished)
            yield job_statuses.get(1, {}), results

        # all jobs finished - final loop
        yield {}, results

    def check_jobs(self, jobid_tasks, results, request_user_input=True):
        finished = []
        for jobid, task in jobid_tasks.items():
            pos = int(os.path.basename(task).split("_")[-1])
            try:
                with lz4.frame.open(os.path.join(task, "result.p.lz4"), 'rb') as f:
                    pickle.load(f)
                results[pos] = os.path.join(task, "result.p.lz4")
                finished.append(jobid)
            except (IOError, EOFError, pickle.UnpicklingError) as e:
                logger.info('Resubmitting {}: {}'.format(jobid, task))
                self.submitter.submit_tasks(
                    [task], start=pos, request_user_input=request_user_input,
                )
                self.submitter.jobid_tasks.pop(jobid)

        return finished

    def query_jobs(self):
        job_status = {}
        out, err = run_command("qstat -g d")

        for l in out.decode('utf-8').splitlines():
            if l.startswith("job-ID") or l.startswith("-----"):
                continue
            ws = l.split()
            jobid = ws[0]
            taskid = int(ws[-1])

            if not '{}.{}'.format(jobid, taskid) in self.submitter.jobid_tasks.keys():
                continue

            state = SGE_JOBSTATE_CODES[ws[4]]
            if state not in job_status:
                job_status[state] = []
            job_status[state].append('{}.{}'.format(jobid, taskid))

        return job_status



import os
from tqdm.auto import tqdm
from .area import WorkingArea
from .submitter import SGETaskSubmitter, MPTaskSubmitter
from .monitor import JobMonitor

def validate_tasks(tasks):
    for task in tasks:
        if any(key not in task for key in ("task", "args", "kwargs")):
            return False
        if not callable(task["task"]):
            return False
    return True

def sge_submit(
    name, path, tasks=[], options="-q hep.q", dryrun=False, quiet=False,
    sleep=5, request_resubmission_options=True,
):
    if not validate_tasks(tasks):
        logger.error(
            "Invalid tasks. Ensure tasks=[{'task': .., 'args': [..], "
            "'kwargs': {..}}, ...], where 'task' is callable."
        )
        return []
    area = WorkingArea(os.path.abspath(path))
    submitter = SGETaskSubmitter(" ".join(['-N {}'.format(name), options]))
    monitor = JobMonitor(submitter)

    results = []
    area.create_areas(tasks, quiet=quiet)
    try:
        submitter.submit_tasks(area.task_paths, dryrun=dryrun, quiet=quiet)
        if not dryrun:
            results = monitor.monitor_jobs(
                sleep=sleep, request_user_input=request_resubmission_options,
            )
    except KeyboardInterrupt as e:
        submitter.killall()
    return results

def sge_submit_yield(
    name, path, tasks=[], options="-q hep.q", quiet=False, sleep=5,
    request_resubmission_options=True,
):
    if not validate_tasks(tasks):
        logger.error(
            "Invalid tasks. Ensure tasks=[{'task': .., 'args': [..], "
            "'kwargs': {..}}, ...], where 'task' is callable."
        )
        return []
    area = WorkingArea(os.path.abspath(path))
    submitter = SGETaskSubmitter(" ".join(['-N {}'.format(name), options]))
    monitor = JobMonitor(submitter)

    area.create_areas(tasks, quiet=quiet)
    submitter.submit_tasks(area.task_paths, quiet=quiet)
    return monitor.request_jobs(
        sleep=sleep, request_user_input=request_resubmission_options,
    )

def sge_resume(
    name, path, options="-q hep.q", quiet=False, sleep=5,
    request_resubmission_options=True,
):
    area = WorkingArea(os.path.abspath(path), resume=True)
    submitter = SGETaskSubmitter(" ".join(['-N {}'.format(name), options]))
    for idx in range(len(area.task_paths)):
        submitter.jobid_tasks['{}'.format(idx)] = area.task_paths[idx]
    monitor = JobMonitor(submitter)

    results = []
    try:
        results = monitor.monitor_jobs(
            sleep=sleep, request_user_input=request_resubmission_options,
        )
    except KeyboardInterrupt as e:
        submitter.killall()
    return results

def mp_submit(tasks, ncores=4, quiet=False):
    if not validate_tasks(tasks):
        logger.error(
            "Invalid tasks. Ensure tasks=[{'task': .., 'args': [..], "
            "'kwargs': {..}}, ...], where 'task' is callable."
        )
        return []
    submitter = MPTaskSubmitter()
    return submitter.submit_tasks(tasks, ncores=ncores, quiet=quiet)

def local_submit(tasks, quiet=False):
    if not validate_tasks(tasks):
        logger.error(
            "Invalid tasks. Ensure tasks=[{'task': .., 'args': [..], "
            "'kwargs': {..}}, ...], where 'task' is callable."
        )
        return []

    results = []
    pbar = tqdm(total=len(tasks), desc="Finished", disable=quiet)

    try:
        for t in tasks:
            results.append(t["task"](*t["args"], **t["kwargs"]))
            pbar.update()
    except KeyboardInterrupt:
        results = []

    pbar.close()
    return results
